'' =================================================================================================
''
''   File....... jm_ra_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS



con

  RA_WINDOW = 5                                                 ' use last 5 samples
  
  
obj

  term : "fullduplexserial64"
 

var

  long  nsamples                                                ' samples taken
  long  raidx                                                   ' current index into samples
  long  samples[RA_WINDOW]                                      ' samples array


dat

TestSamples             long    100, 200, 100, 400, 400, 200, 100, 300, 200, 300


pub main | ra, idx, tsidx

  term.start(RX1, TX1, %0000, 115_200)
  pause(1)

  term.str(string(CLS, "Rolling Average Demo", CR, CR))

  tsidx := 0

  repeat
    ra := rolling_avg(TestSamples[tsidx])                       ' get rolling average

    ' show internals
    
    repeat idx from 0 to RA_WINDOW-1
      term.rjdec(samples[idx], 5, " ")                          ' display samples array
    term.rjdec(nsamples, 7, " ")                                ' show samples used in average
    term.rjdec(ra, 7, " ")                                      ' show average
    term.tx(CR)
    pause(1000)
    
    tsidx := ++tsidx // 10                                      ' keep in range of test samples 


pub rolling_avg(newsample) : out | i

  samples[raidx] := newsample                                   ' add new sample to array
  raidx := ++raidx // RA_WINDOW                                 ' update array index
  nsamples := ++nsamples <# RA_WINDOW                           ' update samples count

  repeat i from 0 to nsamples-1                                 ' loop through active samples
    out += samples[i]                                           ' accumulate

  out /= nsamples                                               ' average 
    

pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)
    

